@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -150,7 +150,8 @@ class MelodicIntervalTeacher(Teacher):
             # Do this loop to make sure two questions in a row does not have
             # the same top or bottom tone.
             while True:
-                self.m_tonika.randomize("f", "f'")
+                self.m_tonika, _ignore = utils.random_tonika_and_interval(L, H,
+                            self.m_question)
                 if not old_tonika:
                     break
                 if old_tonika != self.m_tonika and self.m_tonika + self.m_question[0] != old_toptone:
@@ -165,17 +166,34 @@ class MelodicIntervalTeacher(Teacher):
         self.q_status = self.QSTATUS_NO
         last_tonika = self.m_tonika
         last_question = self.m_question
+        if self.get_bool("lock-to-key"):
+            lock_tonic = mpd.MusicalPitch.new_from_int(self.get_int("lock-to-key-note"))
+            lock_scaletype = solfege.utils.random_tonic_and_interval_in_key.key_data.keys()[:][self.get_int("lock-to-key-scaletype")]
+
         for x in range(10):# we try max 10 times to get a question that
                            # is different from the last one.
-            self.m_tonika, i = utils.random_tonika_and_interval(L, H,
+            if self.get_bool("lock-to-key"):
+                self.m_tonika, i = \
+                    utils.random_tonic_and_interval_in_key(L, H,
+                        self.get_list('ask_for_intervals_0'),
+                        lock_tonic, lock_scaletype)
+            else:
+                self.m_tonika, i = utils.random_tonika_and_interval(L, H,
                             self.get_list('ask_for_intervals_0'))
             self.m_question = [i]
             t = self.m_tonika + i
             for x in range(1, self.get_int('number_of_intervals=1')):
                 if not self.get_list('ask_for_intervals_%i' % x):
                     return self.ERR_CONFIGURE
-                i = utils.random_interval(t, L, H,
-                               self.get_list('ask_for_intervals_%i' % x))
+                if self.get_bool('lock-to-key'):
+                    i = utils.random_interval_in_key(t, L, H,
+                            self.get_list('ask_for_intervals_%i' % x),
+                            lock_tonic, lock_scaletype)
+                    if not i:
+                        return self.ERR_CONFIGURE
+                else:
+                    i = utils.random_interval(t, L, H,
+                            self.get_list('ask_for_intervals_%i' % x))
                 if not i:
                     # if we can't find an interval that is with the range
                     # we, find the interval that is closest to the range
@@ -805,6 +823,28 @@ class IntervalGui(Gui):
         self.g_disable_unused_buttons = gu.nCheckButton(self.m_exname,
                     'disable_unused_intervals', _("_Disable unused buttons"))
         hbox.pack_start(self.g_disable_unused_buttons)
+    def add_lock_to_key_gui(self):
+        # gui to lock to a key
+        def toggle_lock_to_key_sensitivity(checkbutton):
+            self.g_notename.set_sensitive(checkbutton.get_active())
+            self.g_scaletype.set_sensitive(checkbutton.get_active())
+        self.g_lock_to_key_hbox = gtk.HBox()
+        self.config_box.pack_start(self.g_lock_to_key_hbox, False)
+        check = gu.nCheckButton(self.m_exname, 'lock-to-key',
+            _("Lock intervals to key:"),
+            callback = toggle_lock_to_key_sensitivity)
+        self.g_lock_to_key_hbox.pack_start(check, False)
+        self.g_notename = gu.nComboBox(self.m_exname, 'lock-to-key-note',
+            mpd.MusicalPitch.new_from_int(60).get_user_notename(),
+            [mpd.MusicalPitch.new_from_int(60 + x).get_user_notename() for x in range(12)])
+        self.g_notename.show()
+        self.g_lock_to_key_hbox.pack_start(self.g_notename, False)
+        self.g_scaletype = gu.nComboBox(self.m_exname, 'lock-to-key-scaletype', _("Major"),
+            [_("Major"), _("Natural Minor"), _("Harmonic Minor")])
+        self.g_scaletype.show()
+        self.g_lock_to_key_hbox.pack_start(self.g_scaletype, False)
+        self.g_lock_to_key_hbox.set_no_show_all(True)
+        toggle_lock_to_key_sensitivity(check)
     def select_inputwidget(self):
         """
         This will be called by HarmonicInterval and MelodicInterval
